import os

######################################################################
## 
## Rating Policy
## 
######################################################################

class RatingPolicy:
    def Get(self, fname): return 0
    def Put(self, fname, value): pass

## ------------------------------------------------------------------

class SimpleRating(RatingPolicy):
    def Get(self, fname):
        if not os.path.isfile(fname): return 0
        return int(fread(fname))
    def Put(self, fname, value):
        fwrite(fname, value)

## ------------------------------------------------------------------

class SolidRating(RatingPolicy):
    def Get(self, fname):
        if not os.path.isfile(fname): return 0
        ave = FloatingAverage()
        ave.Load(fname)
        return ave.Value()
    def Put(self, fname, value):
        ave = FloatingAverage()
        ave.Load(fname)
        ave.Put(value)
        ave.Save(fname)


######################################################################
## 
## Statistics
## 
######################################################################

class FloatingAverage:
    def __init__(self):
        self.Reset()
    def Reset(self):
        self.n = 0
        self.x = 0.0
        self.y = 0.0
    def Put(self, u):
        self.n += 1
        self.x += u
        self.y = self.x / self.n
    def Value(self):
        return self.y
    def Load(self, fname):
        try:
            s = fread(fname)
            self.n, self.x, self.y = eval(s)
        except IOError:
            self.Reset()
            return False
##         except (SyntaxError, TypeError, ValueError):
##            self.Reset()
##            return False
        else:
            return True
    def Save(self, fname):
        s = `(self.n, self.x, self.y)`
        fwrite(fname, s)


######################################################################
## 
## Auxiliaries
## 
######################################################################

def fread(fname):
    f = open(fname, 'rt')
    try:
        return f.read()
    finally:
        f.close()

def fwrite(fname, content):
    f = open(fname, 'wt')
    try:
        return f.write(str(content))
    finally:
        f.close()

######################################################################
## 
## INIT
## 
######################################################################

##rating = SimpleRating()
rating = SolidRating()
